@@ -16,7 +16,9 @@ pub enum VaultError {
     EntryNotFound,
     /// Invalid AES key length
     InvalidAesKeyLength,
-    /// Invalid HKDF outputtype
+    /// Invalid Secret length
+    InvalidSecretLength,
+    /// Invalid HKDF output type
     InvalidHkdfOutputType,
     /// Invalid private key length
     InvalidPrivateKeyLen,
@@ -36,6 +38,8 @@ pub enum VaultError {
     InvalidBlsSecretLength,
     /// Invalid BLS secret
     InvalidBlsSecret,
+    /// IO error when saving
+    StorageError,
 }
 
 impl VaultError {
@@ -49,7 +53,7 @@ impl From<VaultError> for Error {
     fn from(err: VaultError) -> Self {
         Self::new(
             VaultError::DOMAIN_CODE + (err as u32),
-            VaultError::DOMAIN_NAME,
+            ockam_core::compat::format!("{}::{:?}", module_path!(), err),
         )
     }
 }
