@@ -1,15 +1,14 @@
-use crate::software_vault::SoftwareVault;
+use crate::vault::Vault;
 use crate::VaultError;
-use ockam_core::hex::encode;
 use ockam_core::vault::{Hasher, KeyId, KeyIdVault, PublicKey, Secret};
 use ockam_core::Result;
 use ockam_core::{async_trait, compat::boxed::Box};
 
 #[async_trait]
-impl KeyIdVault for SoftwareVault {
-    async fn secret_by_key_id(&mut self, key_id: &str) -> Result<Secret> {
-        let index = self
-            .entries
+impl KeyIdVault for Vault {
+    async fn secret_by_key_id(&self, key_id: &str) -> Result<Secret> {
+        let entries = self.data.entries.read().await;
+        let index = entries
             .iter()
             .find(|(_, entry)| {
                 if let Some(e_key_id) = entry.key_id() {
@@ -24,18 +23,18 @@ impl KeyIdVault for SoftwareVault {
         Ok(Secret::new(*index))
     }
 
-    async fn compute_key_id_for_public_key(&mut self, public_key: &PublicKey) -> Result<KeyId> {
+    async fn compute_key_id_for_public_key(&self, public_key: &PublicKey) -> Result<KeyId> {
         let key_id = self.sha256(public_key.as_ref()).await?;
-        Ok(encode(key_id))
+        Ok(hex::encode(key_id))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::SoftwareVault;
+    use crate::Vault;
 
-    fn new_vault() -> SoftwareVault {
-        SoftwareVault::default()
+    fn new_vault() -> Vault {
+        Vault::default()
     }
 
     #[ockam_macros::vault_test]
